# SPDX-FileCopyrightText: 2019-2023 Mattia Basaglia <dev@dragon.best>
# SPDX-License-Identifier: GPL-3.0-or-later
def main(window, document, settings):
    window.warning(
        settings["greeting"]
    )
